<?php

return \deepclone_from_array([
    'classes' => 'Symfony\\Component\\VarExporter\\Tests\\FinalError',
    'objectMeta' => [
        [0, 1],
    ],
    'prepared' => 0,
    'properties' => [
        'Error' => [
            'file' => [\dirname(__DIR__).\DIRECTORY_SEPARATOR.'VarExporterTest.php'],
            'line' => [123],
            'trace' => [
                [],
            ],
        ],
    ],
    'states' => [1 => 0],
]);
