<?php

return \deepclone_from_array([
    'classes' => ['SplObjectStorage', 'stdClass'],
    'objectMeta' => [
        [0, -2],
        1,
    ],
    'prepared' => 0,
    'states' => [
        2 => [
            0,
            [
                [1, 345],
                [],
            ],
            [
                [true],
            ],
        ],
    ],
]);
