<?php

return \deepclone_from_array([
    'classes' => 'ArrayIterator',
    'objectMeta' => [
        [0, -1],
    ],
    'prepared' => 0,
    'states' => [
        1 => [
            0,
            [
                1,
                [123],
                [],
                null,
            ],
        ],
    ],
]);
