<?php

return [
    'Names' => [
        'ADP' => [
            'ADP',
            'Андорра песетаһы',
        ],
        'AED' => [
            'AED',
            'Берләшкән Ғәрәп Әмирлектәре дирһәме',
        ],
        'AFA' => [
            'AFA',
            'Афған афғаниһы (1927–2002)',
        ],
        'AFN' => [
            'AFN',
            'Афған афғаниһы',
        ],
        'ALK' => [
            'ALK',
            'Албания лекы (1946–1965)',
        ],
        'ALL' => [
            'ALL',
            'Албания лекы',
        ],
        'AMD' => [
            'AMD',
            'Әрмән драмы',
        ],
        'ANG' => [
            'ANG',
            'Нидерланд Антил гульдены',
        ],
        'AOA' => [
            'AOA',
            'Ангола кванзаһы',
        ],
        'AOK' => [
            'AOK',
            'Ангола кванзаһы (1977–1991)',
        ],
        'AON' => [
            'AON',
            'Ангола яңы кванзаһы (1990–2000)',
        ],
        'AOR' => [
            'AOR',
            'Ангола көйләнгән кванзаһы (1995–1999)',
        ],
        'ARA' => [
            'ARA',
            'Аргентина аустралы',
        ],
        'ARL' => [
            'ARL',
            'Аргентина песоһы (1970–1983)',
        ],
        'ARM' => [
            'ARM',
            'Аргентина песоһы (1881–1970)',
        ],
        'ARP' => [
            'ARP',
            'Аргентина песоһы (1983–1985)',
        ],
        'ARS' => [
            'ARS',
            'Аргентина песоһы',
        ],
        'ATS' => [
            'ATS',
            'Австрия шиллингы',
        ],
        'AUD' => [
            'A$',
            'Австралия доллары',
        ],
        'AWG' => [
            'AWG',
            'Аруба флорины',
        ],
        'AZM' => [
            'AZM',
            'Әзербайжан манаты (1993–2006)',
        ],
        'AZN' => [
            'AZN',
            'Әзербайжан манаты',
        ],
        'BAD' => [
            'BAD',
            'Босния һәм Герцеговина динары (1992–1994)',
        ],
        'BAM' => [
            'BAM',
            'Босния һәм Герцеговина алмашынмалы маркаһы',
        ],
        'BAN' => [
            'BAN',
            'Босния һәм Герцеговина яңы динары (1994–1997)',
        ],
        'BBD' => [
            'BBD',
            'Барбадос доллары',
        ],
        'BDT' => [
            'BDT',
            'Бангладеш такаһы',
        ],
        'BEC' => [
            'BEC',
            'Бельгия франкы (алмашынмалы)',
        ],
        'BEF' => [
            'BEF',
            'Бельгия франкы',
        ],
        'BEL' => [
            'BEL',
            'Бельгия франкы (финанс)',
        ],
        'BGL' => [
            'BGL',
            'Болгар ҡаты левы',
        ],
        'BGM' => [
            'BGM',
            'Болгар социалистик левы',
        ],
        'BGN' => [
            'BGN',
            'Болгария левы',
        ],
        'BGO' => [
            'BGO',
            'Болгар левы (1879–1952)',
        ],
        'BHD' => [
            'BHD',
            'Бәхрәйн динары',
        ],
        'BIF' => [
            'BIF',
            'Бурунди франкы',
        ],
        'BMD' => [
            'BMD',
            'Бермуд доллары',
        ],
        'BND' => [
            'BND',
            'Бруней доллары',
        ],
        'BOB' => [
            'BOB',
            'Боливия боливианоһы',
        ],
        'BOL' => [
            'BOL',
            'Боливия боливианоһы (1863–1963)',
        ],
        'BOP' => [
            'BOP',
            'Боливия песоһы',
        ],
        'BOV' => [
            'BOV',
            'Боливия мвдолы',
        ],
        'BRB' => [
            'BRB',
            'Бразилия яңы крузейроһы (1967–1986)',
        ],
        'BRC' => [
            'BRC',
            'Бразилия крузадоһы (1986–1989)',
        ],
        'BRE' => [
            'BRE',
            'Бразилия крузейроһы (1990–1993)',
        ],
        'BRL' => [
            'R$',
            'Бразилия реалы',
        ],
        'BRN' => [
            'BRN',
            'Бразилия яңы крузадоһы (1989–1990)',
        ],
        'BRR' => [
            'BRR',
            'Бразилия крузейроһы (1993–1994)',
        ],
        'BRZ' => [
            'BRZ',
            'Бразилия крузейроһы (1942–1967)',
        ],
        'BSD' => [
            'BSD',
            'Багам доллары',
        ],
        'BTN' => [
            'BTN',
            'Бутан нгултрумы',
        ],
        'BUK' => [
            'BUK',
            'Бирма кьяты',
        ],
        'BWP' => [
            'BWP',
            'Ботсвана пулаһы',
        ],
        'BYB' => [
            'BYB',
            'Беларусь рубле (1994–1999)',
        ],
        'BYN' => [
            'BYN',
            'Беларусь рубле',
        ],
        'BYR' => [
            'BYR',
            'Беларусь рубле (2000–2016)',
        ],
        'BZD' => [
            'BZD',
            'Белиз доллары',
        ],
        'CAD' => [
            'CA$',
            'Канада доллары',
        ],
        'CDF' => [
            'CDF',
            'Конго франкы',
        ],
        'CHE' => [
            'CHE',
            'WIR евроһы',
        ],
        'CHF' => [
            'CHF',
            'Швейцария франкы',
        ],
        'CHW' => [
            'CHW',
            'WIR франкы',
        ],
        'CLE' => [
            'CLE',
            'Чили эскудоһы',
        ],
        'CLF' => [
            'CLF',
            'Чили иҫәп берәмеге (UF)',
        ],
        'CLP' => [
            'CLP',
            'Чили песоһы',
        ],
        'CNH' => [
            'CNH',
            'Ҡытай юане (офшор)',
        ],
        'CNX' => [
            'CNX',
            'Ҡытай Халыҡ Банкы доллары',
        ],
        'CNY' => [
            'CN¥',
            'Ҡытай юане',
        ],
        'COP' => [
            'COP',
            'Колумбия песоһы',
        ],
        'COU' => [
            'COU',
            'Колумбия реаль хаҡ берәмеге',
        ],
        'CRC' => [
            'CRC',
            'Коста-Рика колоны',
        ],
        'CSD' => [
            'CSD',
            'Сербия динары (2002–2006)',
        ],
        'CSK' => [
            'CSK',
            'Чехословакия ҡаты кронаһы',
        ],
        'CUC' => [
            'CUC',
            'Куба алмашынмалы песоһы',
        ],
        'CUP' => [
            'CUP',
            'Куба песоһы',
        ],
        'CVE' => [
            'CVE',
            'Кабо-Верде эскудоһы',
        ],
        'CYP' => [
            'CYP',
            'Кипр фунты',
        ],
        'CZK' => [
            'CZK',
            'Чехия кронаһы',
        ],
        'DDM' => [
            'DDM',
            'Көнсығыш Германия маркаһы',
        ],
        'DEM' => [
            'DEM',
            'Германия маркаһы',
        ],
        'DJF' => [
            'DJF',
            'Джибути франкы',
        ],
        'DKK' => [
            'DKK',
            'Дания кронаһы',
        ],
        'DOP' => [
            'DOP',
            'Доминикан песоһы',
        ],
        'DZD' => [
            'DZD',
            'Алжир динары',
        ],
        'ECS' => [
            'ECS',
            'Эквадор сукреһы',
        ],
        'ECV' => [
            'ECV',
            'Эквадор даими хаҡ берәмеге',
        ],
        'EEK' => [
            'EEK',
            'Эстония кронаһы',
        ],
        'EGP' => [
            'EGP',
            'Мысыр фунты',
        ],
        'ERN' => [
            'ERN',
            'Эритрея накфаһы',
        ],
        'ESA' => [
            'ESA',
            'Испания песетаһы (A иҫәбе)',
        ],
        'ESB' => [
            'ESB',
            'Испания песетаһы (алмашынмалы иҫәп)',
        ],
        'ESP' => [
            'ESP',
            'Испания песетаһы',
        ],
        'ETB' => [
            'ETB',
            'Эфиопия быры',
        ],
        'EUR' => [
            '€',
            'Евро',
        ],
        'FIM' => [
            'FIM',
            'Финляндия маркаһы',
        ],
        'FJD' => [
            'FJD',
            'Фиджи доллары',
        ],
        'FKP' => [
            'FKP',
            'Фолкленд утрауҙары фунты',
        ],
        'FRF' => [
            'FRF',
            'Франция франкы',
        ],
        'GBP' => [
            '£',
            'Британия фунт стерлингы',
        ],
        'GEK' => [
            'GEK',
            'Грузия купон лариты',
        ],
        'GEL' => [
            'GEL',
            'Грузия лариһы',
        ],
        'GHC' => [
            'GHC',
            'Гана седиһы (1979–2007)',
        ],
        'GHS' => [
            'GHS',
            'Гана седиһы',
        ],
        'GIP' => [
            'GIP',
            'Гибралтар фунты',
        ],
        'GMD' => [
            'GMD',
            'Гамбия даласиһы',
        ],
        'GNF' => [
            'GNF',
            'Гвинея франкы',
        ],
        'GNS' => [
            'GNS',
            'Гвинея силиһы',
        ],
        'GQE' => [
            'GQE',
            'Экваториаль Гвинея эквелеһы',
        ],
        'GRD' => [
            'GRD',
            'Греция драхмаһы',
        ],
        'GTQ' => [
            'GTQ',
            'Гватемала кетсале',
        ],
        'GWE' => [
            'GWE',
            'Португал Гвинея эскудоһы',
        ],
        'GWP' => [
            'GWP',
            'Гвинея-Бисау песоһы',
        ],
        'GYD' => [
            'GYD',
            'Гайана доллары',
        ],
        'HKD' => [
            'HK$',
            'Гонконг доллары',
        ],
        'HNL' => [
            'HNL',
            'Гондурас лемпираһы',
        ],
        'HRD' => [
            'HRD',
            'Хорватия динары',
        ],
        'HRK' => [
            'HRK',
            'Хорватия кунаһы',
        ],
        'HTG' => [
            'HTG',
            'Гаити гурды',
        ],
        'HUF' => [
            'HUF',
            'Венгрия форинты',
        ],
        'IDR' => [
            'IDR',
            'Индонезия рупияһы',
        ],
        'IEP' => [
            'IEP',
            'Ирландия фунты',
        ],
        'ILP' => [
            'ILP',
            'Израиль фунты',
        ],
        'ILR' => [
            'ILR',
            'Израиль шекеле (1980–1985)',
        ],
        'ILS' => [
            '₪',
            'Израиль яңы шекеле',
        ],
        'INR' => [
            '₹',
            'Һиндостан рупияһы',
        ],
        'IQD' => [
            'IQD',
            'Ираҡ динары',
        ],
        'IRR' => [
            'IRR',
            'Иран риалы',
        ],
        'ISJ' => [
            'ISJ',
            'Исландия кронаһы (1918–1981)',
        ],
        'ISK' => [
            'ISK',
            'Исландия кронаһы',
        ],
        'ITL' => [
            'ITL',
            'Италия лираһы',
        ],
        'JMD' => [
            'JMD',
            'Ямайка доллары',
        ],
        'JOD' => [
            'JOD',
            'Иордания динары',
        ],
        'JPY' => [
            'JP¥',
            'Япон иенаһы',
        ],
        'KES' => [
            'KES',
            'Кения шиллингы',
        ],
        'KGS' => [
            'KGS',
            'Ҡырғыҙстан һумы',
        ],
        'KHR' => [
            'KHR',
            'Камбоджа риеле',
        ],
        'KMF' => [
            'KMF',
            'Комор франкы',
        ],
        'KPW' => [
            'KPW',
            'Төньяҡ Корея воны',
        ],
        'KRH' => [
            'KRH',
            'Көньяҡ Корея хваны (1953–1962)',
        ],
        'KRO' => [
            'KRO',
            'Көньяҡ Корея воны (1945–1953)',
        ],
        'KRW' => [
            '₩',
            'Көньяҡ Корея воны',
        ],
        'KWD' => [
            'KWD',
            'Кувейт динары',
        ],
        'KYD' => [
            'KYD',
            'Кайман утрауҙары доллары',
        ],
        'KZT' => [
            'KZT',
            'Ҡаҙағстан тәңкәһе',
        ],
        'LAK' => [
            'LAK',
            'Лаос кипы',
        ],
        'LBP' => [
            'LBP',
            'Ливан фунты',
        ],
        'LKR' => [
            'LKR',
            'Шри-Ланка рупияһы',
        ],
        'LRD' => [
            'LRD',
            'Либерия доллары',
        ],
        'LSL' => [
            'LSL',
            'Лесото лотиһы',
        ],
        'LTL' => [
            'LTL',
            'Латвия литы',
        ],
        'LTT' => [
            'LTT',
            'Литва талоны',
        ],
        'LUC' => [
            'LUC',
            'Люксембург алмашынмалы франкы',
        ],
        'LUF' => [
            'LUF',
            'Люксембург франкы',
        ],
        'LUL' => [
            'LUL',
            'Люксембург финанс франкы',
        ],
        'LVL' => [
            'LVL',
            'Латвия латы',
        ],
        'LVR' => [
            'LVR',
            'Латвия рубле',
        ],
        'LYD' => [
            'LYD',
            'Ливия динары',
        ],
        'MAD' => [
            'MAD',
            'Марокко дирһәме',
        ],
        'MAF' => [
            'MAF',
            'Марокко франкы',
        ],
        'MCF' => [
            'MCF',
            'Монако франкы',
        ],
        'MDC' => [
            'MDC',
            'Молдова купоны',
        ],
        'MDL' => [
            'MDL',
            'Молдова лейе',
        ],
        'MGA' => [
            'MGA',
            'Мадагаскар ариариһы',
        ],
        'MGF' => [
            'MGF',
            'Мадагаскар франкы',
        ],
        'MKD' => [
            'MKD',
            'Македония денары',
        ],
        'MKN' => [
            'MKN',
            'Македония денары (1992–1993)',
        ],
        'MLF' => [
            'MLF',
            'Мали франкы',
        ],
        'MMK' => [
            'MMK',
            'Мьянма кьяты',
        ],
        'MNT' => [
            'MNT',
            'Монголия тугригы',
        ],
        'MOP' => [
            'MOP',
            'Макао патакаһы',
        ],
        'MRO' => [
            'MRO',
            'Мавритания угияһы (1973–2017)',
        ],
        'MRU' => [
            'MRU',
            'Мавритания угияһы',
        ],
        'MTL' => [
            'MTL',
            'Мальта лираһы',
        ],
        'MTP' => [
            'MTP',
            'Мальта фунты',
        ],
        'MUR' => [
            'MUR',
            'Маврикий рупияһы',
        ],
        'MVP' => [
            'MVP',
            'Мальдив рупияһы (1947–1981)',
        ],
        'MVR' => [
            'MVR',
            'Мальдив руфияһы',
        ],
        'MWK' => [
            'MWK',
            'Малави квачаһы',
        ],
        'MXN' => [
            'MX$',
            'Мексика песоһы',
        ],
        'MXP' => [
            'MXP',
            'Мексика көмөш песоһы (1861–1992)',
        ],
        'MXV' => [
            'MXV',
            'Мексика инвестиция берәмеге',
        ],
        'MYR' => [
            'MYR',
            'Малайзия ринггиты',
        ],
        'MZE' => [
            'MZE',
            'Мозамбик эскудоһы',
        ],
        'MZM' => [
            'MZM',
            'Мозамбик метикалы (1980–2006)',
        ],
        'MZN' => [
            'MZN',
            'Мозамбик метикалы',
        ],
        'NAD' => [
            'NAD',
            'Намибия доллары',
        ],
        'NGN' => [
            'NGN',
            'Нигерия найраһы',
        ],
        'NIC' => [
            'NIC',
            'Никарагуа кордобаһы (1988–1991)',
        ],
        'NIO' => [
            'NIO',
            'Никарагуа кордобаһы',
        ],
        'NLG' => [
            'NLG',
            'Нидерланд гульдены',
        ],
        'NOK' => [
            'NOK',
            'Норвегия кронаһы',
        ],
        'NPR' => [
            'NPR',
            'Непал рупияһы',
        ],
        'NZD' => [
            'NZ$',
            'Яңы Зеландия доллары',
        ],
        'OMR' => [
            'OMR',
            'Оман риалы',
        ],
        'PAB' => [
            'PAB',
            'Панама бальбоаһы',
        ],
        'PEI' => [
            'PEI',
            'Перу интиһы',
        ],
        'PEN' => [
            'PEN',
            'Перу соле',
        ],
        'PES' => [
            'PES',
            'Перу солеһы (1863–1965)',
        ],
        'PGK' => [
            'PGK',
            'Папуа - Яңы Гвинея кинаһы',
        ],
        'PHP' => [
            '₱',
            'Филиппин песоһы',
        ],
        'PKR' => [
            'PKR',
            'Пакистан рупияһы',
        ],
        'PLN' => [
            'PLN',
            'Польша злотыйы',
        ],
        'PLZ' => [
            'PLZ',
            'Польша злотыйы (1950–1995)',
        ],
        'PTE' => [
            'PTE',
            'Португалия эскудоһы',
        ],
        'PYG' => [
            'PYG',
            'Парагвай гуараниһы',
        ],
        'QAR' => [
            'QAR',
            'Катар риалы',
        ],
        'RHD' => [
            'RHD',
            'Родезия доллары',
        ],
        'ROL' => [
            'ROL',
            'Румын лейе (1952–2006)',
        ],
        'RON' => [
            'RON',
            'Румын лейе',
        ],
        'RSD' => [
            'RSD',
            'Сербия динары',
        ],
        'RUB' => [
            'RUB',
            'Рәсәй рубле',
        ],
        'RUR' => [
            'р.',
            'Рәсәй рубле (1991–1998)',
        ],
        'RWF' => [
            'RWF',
            'Руанда франкы',
        ],
        'SAR' => [
            'SAR',
            'Сәғүд риалы',
        ],
        'SBD' => [
            'SBD',
            'Соломон утрауҙары доллары',
        ],
        'SCR' => [
            'SCR',
            'Сейшел рупияһы',
        ],
        'SDD' => [
            'SDD',
            'Судан динары (1992–2007)',
        ],
        'SDG' => [
            'SDG',
            'Судан фунты',
        ],
        'SDP' => [
            'SDP',
            'Судан фунты (1957–1998)',
        ],
        'SEK' => [
            'SEK',
            'Швеция кронаһы',
        ],
        'SGD' => [
            'SGD',
            'Сингапур доллары',
        ],
        'SHP' => [
            'SHP',
            'Изге Елена утрауы фунты',
        ],
        'SIT' => [
            'SIT',
            'Словения толары',
        ],
        'SKK' => [
            'SKK',
            'Словакия кронаһы',
        ],
        'SLE' => [
            'SLE',
            'Сьерра-Леоне леонеһы',
        ],
        'SLL' => [
            'SLL',
            'Сьерра-Леоне леонеһы (1964–2022)',
        ],
        'SOS' => [
            'SOS',
            'Сомали шиллингы',
        ],
        'SRD' => [
            'SRD',
            'Суринам доллары',
        ],
        'SRG' => [
            'SRG',
            'Суринам гульдены',
        ],
        'SSP' => [
            'SSP',
            'Көньяҡ Судан фунты',
        ],
        'STD' => [
            'STD',
            'Сан-Томе һәм Принсипи добраһы (1977–2017)',
        ],
        'STN' => [
            'STN',
            'Сан-Томе һәм Принсипи добраһы',
        ],
        'SUR' => [
            'SUR',
            'СССР рубле',
        ],
        'SVC' => [
            'SVC',
            'Сальвадор колоны',
        ],
        'SYP' => [
            'SYP',
            'Сүриә фунты',
        ],
        'SZL' => [
            'SZL',
            'Свазиленд лилангениһы',
        ],
        'THB' => [
            'THB',
            'Таиланд баты',
        ],
        'TJR' => [
            'TJR',
            'Тажикстан рубле',
        ],
        'TJS' => [
            'TJS',
            'Тажикстан сомониһы',
        ],
        'TMM' => [
            'TMM',
            'Төркмәнстан манаты (1993–2009)',
        ],
        'TMT' => [
            'TMT',
            'Төркмәнстан манаты',
        ],
        'TND' => [
            'TND',
            'Тунис динары',
        ],
        'TOP' => [
            'TOP',
            'Тонга паангаһы',
        ],
        'TPE' => [
            'TPE',
            'Тимор эскудоһы',
        ],
        'TRL' => [
            'TRL',
            'Төрөк лираһы (1922–2005)',
        ],
        'TRY' => [
            'TRY',
            'Төрөк лираһы',
        ],
        'TTD' => [
            'TTD',
            'Тринидад һәм Тобаго доллары',
        ],
        'TWD' => [
            'NT$',
            'Яңы Тайвань доллары',
        ],
        'TZS' => [
            'TZS',
            'Танзания шиллингы',
        ],
        'UAH' => [
            'UAH',
            'Украина гривнаһы',
        ],
        'UAK' => [
            'UAK',
            'Украина карбованецы',
        ],
        'UGS' => [
            'UGS',
            'Уганда шиллингы (1966–1987)',
        ],
        'UGX' => [
            'UGX',
            'Уганда шиллингы',
        ],
        'USD' => [
            '$',
            'АҠШ доллары',
        ],
        'USN' => [
            'USN',
            'АҠШ доллары (киләһе көн)',
        ],
        'USS' => [
            'USS',
            'АҠШ доллары (шул уҡ көн)',
        ],
        'UYI' => [
            'UYI',
            'Уругвай песоһы (индексланған берәмектәр)',
        ],
        'UYP' => [
            'UYP',
            'Уругвай песоһы (1975–1993)',
        ],
        'UYU' => [
            'UYU',
            'Уругвай песоһы',
        ],
        'UYW' => [
            'UYW',
            'Уругвай номиналь эш хаҡы индексы берәмеге',
        ],
        'UZS' => [
            'UZS',
            'Үзбәкстан һумы',
        ],
        'VEB' => [
            'VEB',
            'Венесуэла боливары (1871–2008)',
        ],
        'VED' => [
            'VED',
            'суверенлы боливар',
        ],
        'VEF' => [
            'VEF',
            'Венесуэла боливары (2008–2018)',
        ],
        'VES' => [
            'VES',
            'Венесуэла боливары',
        ],
        'VND' => [
            '₫',
            'Вьетнам донгы',
        ],
        'VNN' => [
            'VNN',
            'Вьетнам донгы (1978–1985)',
        ],
        'VUV' => [
            'VUV',
            'Вануату ватуһы',
        ],
        'WST' => [
            'WST',
            'Самоа талаһы',
        ],
        'XAF' => [
            'FCFA',
            'Үҙәк Африка КФА франкы',
        ],
        'XCD' => [
            'EC$',
            'Көнсығыш Кариб доллары',
        ],
        'XCG' => [
            'Cg.',
            'Кариб гульдены',
        ],
        'XEU' => [
            'XEU',
            'Европа валюта берәмеге',
        ],
        'XFO' => [
            'XFO',
            'Франция алтын франкы',
        ],
        'XFU' => [
            'XFU',
            'Франция UIC-франкы',
        ],
        'XOF' => [
            'F CFA',
            'Көнбайыш Африка КФА франкы',
        ],
        'XPF' => [
            'CFPF',
            'Француз Тымыҡ океан франкы',
        ],
        'XRE' => [
            'XRE',
            'RINET фондтары',
        ],
        'YDD' => [
            'YDD',
            'Йәмән динары',
        ],
        'YER' => [
            'YER',
            'Йәмән риалы',
        ],
        'YUD' => [
            'YUD',
            'Югославия ҡаты динары (1966–1990)',
        ],
        'YUM' => [
            'YUM',
            'Югославия яңы динары (1994–2002)',
        ],
        'YUN' => [
            'YUN',
            'Югославия алмашынмалы динары (1990–1992)',
        ],
        'YUR' => [
            'YUR',
            'Югославия реформаланған динары (1992–1993)',
        ],
        'ZAL' => [
            'ZAL',
            'Көньяҡ Африка рэнды (финанс)',
        ],
        'ZAR' => [
            'ZAR',
            'Көньяҡ Африка рэнды',
        ],
        'ZMK' => [
            'ZMK',
            'Замбия квачаһы (1968–2012)',
        ],
        'ZMW' => [
            'ZMW',
            'Замбия квачаһы',
        ],
        'ZRN' => [
            'ZRN',
            'Заир яңы заиры (1993–1998)',
        ],
        'ZRZ' => [
            'ZRZ',
            'Заир заиры (1971–1993)',
        ],
        'ZWD' => [
            'ZWD',
            'Зимбабве доллары (1980–2008)',
        ],
        'ZWG' => [
            'ZWG',
            'Зимбабве алтыны',
        ],
        'ZWL' => [
            'ZWL',
            'Зимбабве доллары (2009–2024)',
        ],
        'ZWR' => [
            'ZWR',
            'Зимбабве доллары (2008)',
        ],
    ],
];
