<?php

return \deepclone_from_array([
    'classes' => 'Symfony\\Component\\VarExporter\\Tests\\Fixtures\\BackedProperty',
    'objectMeta' => 1,
    'prepared' => 0,
    'properties' => [
        'Symfony\\Component\\VarExporter\\Tests\\Fixtures\\BackedProperty' => [
            'name' => ['name'],
        ],
    ],
]);
