<?php

namespace Symfony\Component\ErrorHandler\Tests\Fixtures;

/**
 * @method string abstractClassMethod()
 * @method static \stdClass abstractStaticMethod() Description
 */
abstract class VirtualAbstract
{
}
