<?php

return \deepclone_from_array([
    'classes' => 'Symfony\\Component\\VarExporter\\Tests\\Fixtures\\GoodNight',
    'objectMeta' => 1,
    'prepared' => 0,
    'properties' => [
        'stdClass' => [
            'good' => ['night'],
        ],
        'Symfony\\Component\\VarExporter\\Tests\\Fixtures\\GoodNight' => [
            'foo' => ['afternoon'],
            'bar' => ['morning'],
        ],
    ],
]);
