<?php

return \deepclone_from_array([
    'classes' => '',
    'objectMeta' => 0,
    'prepared' => [
        ['Symfony\\Component\\VarExporter\\Tests\\Fixtures\\PrivateFCC', 'testMethod'],
        'Symfony\\Component\\VarExporter\\Tests\\Fixtures\\PrivateFCC',
        'testMethod',
    ],
    'mask' => 0,
]);
