<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Bundle;

/**
 * A marker to be able to check if symfony/symfony is installed instead of the individual components/bundles.
 *
 * @internal
 */
final class FullStack
{
}
