<?php

return [
    'Names' => [
        'af' => 'африкаанс',
        'af_NA' => 'африкаанс (Намибия)',
        'af_ZA' => 'африкаанс (Көньяҡ Африка Республикаһы)',
        'ak' => 'акан',
        'ak_GH' => 'акан (Гана)',
        'am' => 'амхар',
        'am_ET' => 'амхар (Эфиопия)',
        'ar' => 'ғәрәп',
        'ar_001' => 'ғәрәп (донъя)',
        'ar_AE' => 'ғәрәп (Берләшкән Ғәрәп Әмирлектәре)',
        'ar_BH' => 'ғәрәп (Бәхрәйн)',
        'ar_DJ' => 'ғәрәп (Джибути)',
        'ar_DZ' => 'ғәрәп (Алжир)',
        'ar_EG' => 'ғәрәп (Мысыр)',
        'ar_EH' => 'ғәрәп (Көнбайыш Сахара)',
        'ar_ER' => 'ғәрәп (Эритрея)',
        'ar_IL' => 'ғәрәп (Израиль)',
        'ar_IQ' => 'ғәрәп (Ираҡ)',
        'ar_JO' => 'ғәрәп (Иордания)',
        'ar_KM' => 'ғәрәп (Комор утрауҙары)',
        'ar_KW' => 'ғәрәп (Кувейт)',
        'ar_LB' => 'ғәрәп (Ливан)',
        'ar_LY' => 'ғәрәп (Ливия)',
        'ar_MA' => 'ғәрәп (Марокко)',
        'ar_MR' => 'ғәрәп (Мавритания)',
        'ar_OM' => 'ғәрәп (Оман)',
        'ar_PS' => 'ғәрәп (Фәләстин территориялары)',
        'ar_QA' => 'ғәрәп (Катар)',
        'ar_SA' => 'ғәрәп (Сәғүд Ғәрәбстаны)',
        'ar_SD' => 'ғәрәп (Судан)',
        'ar_SO' => 'ғәрәп (Сомали)',
        'ar_SS' => 'ғәрәп (Көньяҡ Судан)',
        'ar_SY' => 'ғәрәп (Сүриә)',
        'ar_TD' => 'ғәрәп (Чад)',
        'ar_TN' => 'ғәрәп (Тунис)',
        'ar_YE' => 'ғәрәп (Йәмән)',
        'as' => 'ассам',
        'as_IN' => 'ассам (Һиндостан)',
        'az' => 'әзербайжан',
        'az_AZ' => 'әзербайжан (Әзербайжан)',
        'az_Cyrl' => 'әзербайжан (кириллица)',
        'az_Cyrl_AZ' => 'әзербайжан (кириллица, Әзербайжан)',
        'az_Latn' => 'әзербайжан (латиница)',
        'az_Latn_AZ' => 'әзербайжан (латиница, Әзербайжан)',
        'ba' => 'башҡорт',
        'ba_RU' => 'башҡорт (Рәсәй)',
        'be' => 'белорус',
        'be_BY' => 'белорус (Беларусь)',
        'bg' => 'болгар',
        'bg_BG' => 'болгар (Болгария)',
        'bm' => 'бамбара',
        'bm_ML' => 'бамбара (Мали)',
        'bn' => 'бенгал',
        'bn_BD' => 'бенгал (Бангладеш)',
        'bn_IN' => 'бенгал (Һиндостан)',
        'bo' => 'тибет',
        'bo_CN' => 'тибет (Ҡытай)',
        'bo_IN' => 'тибет (Һиндостан)',
        'br' => 'бретон',
        'br_FR' => 'бретон (Франция)',
        'bs' => 'босний',
        'bs_BA' => 'босний (Босния һәм Герцеговина)',
        'bs_Cyrl' => 'босний (кириллица)',
        'bs_Cyrl_BA' => 'босний (кириллица, Босния һәм Герцеговина)',
        'bs_Latn' => 'босний (латиница)',
        'bs_Latn_BA' => 'босний (латиница, Босния һәм Герцеговина)',
        'ca' => 'каталан',
        'ca_AD' => 'каталан (Андорра)',
        'ca_ES' => 'каталан (Испания)',
        'ca_FR' => 'каталан (Франция)',
        'ca_IT' => 'каталан (Италия)',
        'ce' => 'чечен',
        'ce_RU' => 'чечен (Рәсәй)',
        'cs' => 'чех',
        'cs_CZ' => 'чех (Чехия)',
        'cv' => 'сыуаш',
        'cv_RU' => 'сыуаш (Рәсәй)',
        'cy' => 'валлий',
        'cy_GB' => 'валлий (Бөйөк Британия)',
        'da' => 'дат',
        'da_DK' => 'дат (Дания)',
        'da_GL' => 'дат (Гренландия)',
        'de' => 'немец',
        'de_AT' => 'немец (Австрия)',
        'de_BE' => 'немец (Бельгия)',
        'de_CH' => 'немец (Швейцария)',
        'de_DE' => 'немец (Германия)',
        'de_IT' => 'немец (Италия)',
        'de_LI' => 'немец (Лихтенштейн)',
        'de_LU' => 'немец (Люксембург)',
        'dz' => 'дзонг-кэ',
        'dz_BT' => 'дзонг-кэ (Бутан)',
        'ee' => 'эве',
        'ee_GH' => 'эве (Гана)',
        'ee_TG' => 'эве (Того)',
        'el' => 'грек',
        'el_CY' => 'грек (Кипр)',
        'el_GR' => 'грек (Греция)',
        'en' => 'инглиз',
        'en_001' => 'инглиз (донъя)',
        'en_150' => 'инглиз (Европа)',
        'en_AE' => 'инглиз (Берләшкән Ғәрәп Әмирлектәре)',
        'en_AG' => 'инглиз (Антигуа һәм Барбуда)',
        'en_AI' => 'инглиз (Ангилья)',
        'en_AS' => 'инглиз (Америка Самоаһы)',
        'en_AT' => 'инглиз (Австрия)',
        'en_AU' => 'инглиз (Австралия)',
        'en_BB' => 'инглиз (Барбадос)',
        'en_BE' => 'инглиз (Бельгия)',
        'en_BI' => 'инглиз (Бурунди)',
        'en_BM' => 'инглиз (Бермуд утрауҙары)',
        'en_BS' => 'инглиз (Багам утрауҙары)',
        'en_BW' => 'инглиз (Ботсвана)',
        'en_BZ' => 'инглиз (Белиз)',
        'en_CA' => 'инглиз (Канада)',
        'en_CC' => 'инглиз (Кокос утрауҙары)',
        'en_CH' => 'инглиз (Швейцария)',
        'en_CK' => 'инглиз (Кук утрауҙары)',
        'en_CM' => 'инглиз (Камерун)',
        'en_CX' => 'инглиз (Раштыуа Утрауы)',
        'en_CY' => 'инглиз (Кипр)',
        'en_CZ' => 'инглиз (Чехия)',
        'en_DE' => 'инглиз (Германия)',
        'en_DK' => 'инглиз (Дания)',
        'en_DM' => 'инглиз (Доминика)',
        'en_EE' => 'инглиз (Эстония)',
        'en_ER' => 'инглиз (Эритрея)',
        'en_ES' => 'инглиз (Испания)',
        'en_FI' => 'инглиз (Финляндия)',
        'en_FJ' => 'инглиз (Фиджи)',
        'en_FK' => 'инглиз (Фолкленд утрауҙары)',
        'en_FM' => 'инглиз (Микронезия)',
        'en_FR' => 'инглиз (Франция)',
        'en_GB' => 'инглиз (Бөйөк Британия)',
        'en_GD' => 'инглиз (Гренада)',
        'en_GE' => 'инглиз (Грузия)',
        'en_GG' => 'инглиз (Гернси)',
        'en_GH' => 'инглиз (Гана)',
        'en_GI' => 'инглиз (Гибралтар)',
        'en_GM' => 'инглиз (Гамбия)',
        'en_GS' => 'инглиз (Көньяҡ Георгия һәм Көньяҡ Сандвич утрауҙары)',
        'en_GU' => 'инглиз (Гуам)',
        'en_GY' => 'инглиз (Гайана)',
        'en_HK' => 'инглиз (Гонконг, Ҡытай МАР-ы)',
        'en_HU' => 'инглиз (Венгрия)',
        'en_ID' => 'инглиз (Индонезия)',
        'en_IE' => 'инглиз (Ирландия)',
        'en_IL' => 'инглиз (Израиль)',
        'en_IM' => 'инглиз (Мэн утрауы)',
        'en_IN' => 'инглиз (Һиндостан)',
        'en_IO' => 'инглиз (Британияның Һинд океанындағы биләмәһе)',
        'en_IT' => 'инглиз (Италия)',
        'en_JE' => 'инглиз (Джерси)',
        'en_JM' => 'инглиз (Ямайка)',
        'en_JP' => 'инглиз (Япония)',
        'en_KE' => 'инглиз (Кения)',
        'en_KI' => 'инглиз (Кирибати)',
        'en_KN' => 'инглиз (Сент-Китс һәм Невис)',
        'en_KY' => 'инглиз (Кайман утрауҙары)',
        'en_LC' => 'инглиз (Сент-Люсия)',
        'en_LR' => 'инглиз (Либерия)',
        'en_LS' => 'инглиз (Лесото)',
        'en_LT' => 'инглиз (Литва)',
        'en_LV' => 'инглиз (Латвия)',
        'en_MG' => 'инглиз (Мадагаскар)',
        'en_MH' => 'инглиз (Маршалл утрауҙары)',
        'en_MO' => 'инглиз (Макао, Ҡытай МАР-ы)',
        'en_MP' => 'инглиз (Төньяҡ Мариан утрауҙары)',
        'en_MS' => 'инглиз (Монтсеррат)',
        'en_MT' => 'инглиз (Мальта)',
        'en_MU' => 'инглиз (Маврикий)',
        'en_MV' => 'инглиз (Мальдив утрауҙары)',
        'en_MW' => 'инглиз (Малави)',
        'en_MY' => 'инглиз (Малайзия)',
        'en_NA' => 'инглиз (Намибия)',
        'en_NF' => 'инглиз (Норфолк утрауы)',
        'en_NG' => 'инглиз (Нигерия)',
        'en_NL' => 'инглиз (Нидерланд)',
        'en_NO' => 'инглиз (Норвегия)',
        'en_NR' => 'инглиз (Науру)',
        'en_NU' => 'инглиз (Ниуэ)',
        'en_NZ' => 'инглиз (Яңы Зеландия)',
        'en_PG' => 'инглиз (Папуа – Яңы Гвинея)',
        'en_PH' => 'инглиз (Филиппин)',
        'en_PK' => 'инглиз (Пакистан)',
        'en_PL' => 'инглиз (Польша)',
        'en_PN' => 'инглиз (Питкэрн утрауҙары)',
        'en_PR' => 'инглиз (Пуэрто-Рико)',
        'en_PT' => 'инглиз (Португалия)',
        'en_PW' => 'инглиз (Палау)',
        'en_RO' => 'инглиз (Румыния)',
        'en_RW' => 'инглиз (Руанда)',
        'en_SB' => 'инглиз (Соломон утрауҙары)',
        'en_SC' => 'инглиз (Сейшель утрауҙары)',
        'en_SD' => 'инглиз (Судан)',
        'en_SE' => 'инглиз (Швеция)',
        'en_SG' => 'инглиз (Сингапур)',
        'en_SH' => 'инглиз (Изге Елена утрауы)',
        'en_SI' => 'инглиз (Словения)',
        'en_SK' => 'инглиз (Словакия)',
        'en_SL' => 'инглиз (Сьерра-Леоне)',
        'en_SS' => 'инглиз (Көньяҡ Судан)',
        'en_SX' => 'инглиз (Синт-Мартен)',
        'en_SZ' => 'инглиз (Эсватини)',
        'en_TC' => 'инглиз (Төркс һәм Кайкос утрауҙары)',
        'en_TK' => 'инглиз (Токелау)',
        'en_TO' => 'инглиз (Тонга)',
        'en_TT' => 'инглиз (Тринидад һәм Тобаго)',
        'en_TV' => 'инглиз (Тувалу)',
        'en_TZ' => 'инглиз (Танзания)',
        'en_UA' => 'инглиз (Украина)',
        'en_UG' => 'инглиз (Уганда)',
        'en_UM' => 'инглиз (АҠШ-ның ситтәге утрауҙары)',
        'en_US' => 'инглиз (Америка Ҡушма Штаттары)',
        'en_VC' => 'инглиз (Сент-Винсент һәм Гренадиндар)',
        'en_VG' => 'инглиз (Британия Виргин утрауҙары)',
        'en_VI' => 'инглиз (АҠШ Виргин утрауҙары)',
        'en_VU' => 'инглиз (Вануату)',
        'en_WS' => 'инглиз (Самоа)',
        'en_ZA' => 'инглиз (Көньяҡ Африка Республикаһы)',
        'en_ZM' => 'инглиз (Замбия)',
        'en_ZW' => 'инглиз (Зимбабве)',
        'eo' => 'эсперанто',
        'eo_001' => 'эсперанто (донъя)',
        'es' => 'испан',
        'es_419' => 'испан (Латин Америка)',
        'es_AR' => 'испан (Аргентина)',
        'es_BO' => 'испан (Боливия)',
        'es_BR' => 'испан (Бразилия)',
        'es_BZ' => 'испан (Белиз)',
        'es_CL' => 'испан (Чили)',
        'es_CO' => 'испан (Колумбия)',
        'es_CR' => 'испан (Коста-Рика)',
        'es_CU' => 'испан (Куба)',
        'es_DO' => 'испан (Доминикан Республикаһы)',
        'es_EC' => 'испан (Эквадор)',
        'es_ES' => 'испан (Испания)',
        'es_GQ' => 'испан (Экваториаль Гвинея)',
        'es_GT' => 'испан (Гватемала)',
        'es_HN' => 'испан (Гондурас)',
        'es_MX' => 'испан (Мексика)',
        'es_NI' => 'испан (Никарагуа)',
        'es_PA' => 'испан (Панама)',
        'es_PE' => 'испан (Перу)',
        'es_PH' => 'испан (Филиппин)',
        'es_PR' => 'испан (Пуэрто-Рико)',
        'es_PY' => 'испан (Парагвай)',
        'es_SV' => 'испан (Сальвадор)',
        'es_US' => 'испан (Америка Ҡушма Штаттары)',
        'es_UY' => 'испан (Уругвай)',
        'es_VE' => 'испан (Венесуэла)',
        'et' => 'эстон',
        'et_EE' => 'эстон (Эстония)',
        'eu' => 'баск',
        'eu_ES' => 'баск (Испания)',
        'fa' => 'фарсы',
        'fa_AF' => 'фарсы (Афғанстан)',
        'fa_IR' => 'фарсы (Иран)',
        'ff' => 'фула',
        'ff_Adlm' => 'фула (адлам)',
        'ff_Adlm_BF' => 'фула (адлам, Буркина-Фасо)',
        'ff_Adlm_CM' => 'фула (адлам, Камерун)',
        'ff_Adlm_GH' => 'фула (адлам, Гана)',
        'ff_Adlm_GM' => 'фула (адлам, Гамбия)',
        'ff_Adlm_GN' => 'фула (адлам, Гвинея)',
        'ff_Adlm_GW' => 'фула (адлам, Гвинея-Бисау)',
        'ff_Adlm_LR' => 'фула (адлам, Либерия)',
        'ff_Adlm_MR' => 'фула (адлам, Мавритания)',
        'ff_Adlm_NE' => 'фула (адлам, Нигер)',
        'ff_Adlm_NG' => 'фула (адлам, Нигерия)',
        'ff_Adlm_SL' => 'фула (адлам, Сьерра-Леоне)',
        'ff_Adlm_SN' => 'фула (адлам, Сенегал)',
        'ff_CM' => 'фула (Камерун)',
        'ff_GN' => 'фула (Гвинея)',
        'ff_Latn' => 'фула (латиница)',
        'ff_Latn_BF' => 'фула (латиница, Буркина-Фасо)',
        'ff_Latn_CM' => 'фула (латиница, Камерун)',
        'ff_Latn_GH' => 'фула (латиница, Гана)',
        'ff_Latn_GM' => 'фула (латиница, Гамбия)',
        'ff_Latn_GN' => 'фула (латиница, Гвинея)',
        'ff_Latn_GW' => 'фула (латиница, Гвинея-Бисау)',
        'ff_Latn_LR' => 'фула (латиница, Либерия)',
        'ff_Latn_MR' => 'фула (латиница, Мавритания)',
        'ff_Latn_NE' => 'фула (латиница, Нигер)',
        'ff_Latn_NG' => 'фула (латиница, Нигерия)',
        'ff_Latn_SL' => 'фула (латиница, Сьерра-Леоне)',
        'ff_Latn_SN' => 'фула (латиница, Сенегал)',
        'ff_MR' => 'фула (Мавритания)',
        'ff_SN' => 'фула (Сенегал)',
        'fi' => 'фин',
        'fi_FI' => 'фин (Финляндия)',
        'fo' => 'фарер',
        'fo_DK' => 'фарер (Дания)',
        'fo_FO' => 'фарер (Фарер утрауҙары)',
        'fr' => 'француз',
        'fr_BE' => 'француз (Бельгия)',
        'fr_BF' => 'француз (Буркина-Фасо)',
        'fr_BI' => 'француз (Бурунди)',
        'fr_BJ' => 'француз (Бенин)',
        'fr_BL' => 'француз (Сен-Бартелеми)',
        'fr_CA' => 'француз (Канада)',
        'fr_CD' => 'француз (Конго - Киншаса)',
        'fr_CF' => 'француз (Үҙәк Африка Республикаһы)',
        'fr_CG' => 'француз (Конго - Браззавиль)',
        'fr_CH' => 'француз (Швейцария)',
        'fr_CI' => 'француз (Кот-д’Ивуар)',
        'fr_CM' => 'француз (Камерун)',
        'fr_DJ' => 'француз (Джибути)',
        'fr_DZ' => 'француз (Алжир)',
        'fr_FR' => 'француз (Франция)',
        'fr_GA' => 'француз (Габон)',
        'fr_GF' => 'француз (Француз Гвианаһы)',
        'fr_GN' => 'француз (Гвинея)',
        'fr_GP' => 'француз (Гваделупа)',
        'fr_GQ' => 'француз (Экваториаль Гвинея)',
        'fr_HT' => 'француз (Гаити)',
        'fr_KM' => 'француз (Комор утрауҙары)',
        'fr_LU' => 'француз (Люксембург)',
        'fr_MA' => 'француз (Марокко)',
        'fr_MC' => 'француз (Монако)',
        'fr_MF' => 'француз (Сен-Мартен)',
        'fr_MG' => 'француз (Мадагаскар)',
        'fr_ML' => 'француз (Мали)',
        'fr_MQ' => 'француз (Мартиника)',
        'fr_MR' => 'француз (Мавритания)',
        'fr_MU' => 'француз (Маврикий)',
        'fr_NC' => 'француз (Яңы Каледония)',
        'fr_NE' => 'француз (Нигер)',
        'fr_PF' => 'француз (Француз Полинезияһы)',
        'fr_PM' => 'француз (Сен-Пьер һәм Микелон)',
        'fr_RE' => 'француз (Реюньон)',
        'fr_RW' => 'француз (Руанда)',
        'fr_SC' => 'француз (Сейшель утрауҙары)',
        'fr_SN' => 'француз (Сенегал)',
        'fr_SY' => 'француз (Сүриә)',
        'fr_TD' => 'француз (Чад)',
        'fr_TG' => 'француз (Того)',
        'fr_TN' => 'француз (Тунис)',
        'fr_VU' => 'француз (Вануату)',
        'fr_WF' => 'француз (Уоллис һәм Футуна)',
        'fr_YT' => 'француз (Майотта)',
        'fy' => 'көнбайыш фриз',
        'fy_NL' => 'көнбайыш фриз (Нидерланд)',
        'ga' => 'ирланд',
        'ga_GB' => 'ирланд (Бөйөк Британия)',
        'ga_IE' => 'ирланд (Ирландия)',
        'gd' => 'шотланд гэл',
        'gd_GB' => 'шотланд гэл (Бөйөк Британия)',
        'gl' => 'галисий',
        'gl_ES' => 'галисий (Испания)',
        'gu' => 'гуджарати',
        'gu_IN' => 'гуджарати (Һиндостан)',
        'gv' => 'мэн',
        'gv_IM' => 'мэн (Мэн утрауы)',
        'ha' => 'хауса',
        'ha_GH' => 'хауса (Гана)',
        'ha_NE' => 'хауса (Нигер)',
        'ha_NG' => 'хауса (Нигерия)',
        'he' => 'иврит',
        'he_IL' => 'иврит (Израиль)',
        'hi' => 'һинди',
        'hi_IN' => 'һинди (Һиндостан)',
        'hi_Latn' => 'һинди (латиница)',
        'hi_Latn_IN' => 'һинди (латиница, Һиндостан)',
        'hr' => 'хорват',
        'hr_BA' => 'хорват (Босния һәм Герцеговина)',
        'hr_HR' => 'хорват (Хорватия)',
        'hu' => 'венгр',
        'hu_HU' => 'венгр (Венгрия)',
        'hy' => 'әрмән',
        'hy_AM' => 'әрмән (Әрмәнстан)',
        'ia' => 'интерлингва',
        'ia_001' => 'интерлингва (донъя)',
        'id' => 'индонез',
        'id_ID' => 'индонез (Индонезия)',
        'ie' => 'интерлингве',
        'ie_EE' => 'интерлингве (Эстония)',
        'ig' => 'игбо',
        'ig_NG' => 'игбо (Нигерия)',
        'ii' => 'сычуань и',
        'ii_CN' => 'сычуань и (Ҡытай)',
        'is' => 'исланд',
        'is_IS' => 'исланд (Исландия)',
        'it' => 'итальян',
        'it_CH' => 'итальян (Швейцария)',
        'it_IT' => 'итальян (Италия)',
        'it_SM' => 'итальян (Сан-Марино)',
        'it_VA' => 'итальян (Ватикан)',
        'ja' => 'япон',
        'ja_JP' => 'япон (Япония)',
        'jv' => 'ява',
        'jv_ID' => 'ява (Индонезия)',
        'ka' => 'грузин',
        'ka_GE' => 'грузин (Грузия)',
        'ki' => 'кикуйю',
        'ki_KE' => 'кикуйю (Кения)',
        'kk' => 'ҡаҙаҡ',
        'kk_Arab' => 'ҡаҙаҡ (ғәрәп)',
        'kk_Arab_CN' => 'ҡаҙаҡ (ғәрәп, Ҡытай)',
        'kk_Cyrl' => 'ҡаҙаҡ (кириллица)',
        'kk_Cyrl_KZ' => 'ҡаҙаҡ (кириллица, Ҡаҙағстан)',
        'kk_KZ' => 'ҡаҙаҡ (Ҡаҙағстан)',
        'kl' => 'калааллисут',
        'kl_GL' => 'калааллисут (Гренландия)',
        'km' => 'кхмер',
        'km_KH' => 'кхмер (Камбоджа)',
        'kn' => 'каннада',
        'kn_IN' => 'каннада (Һиндостан)',
        'ko' => 'корей',
        'ko_CN' => 'корей (Ҡытай)',
        'ko_KP' => 'корей (Төньяҡ Корея)',
        'ko_KR' => 'корей (Көньяҡ Корея)',
        'ks' => 'кашмири',
        'ks_Arab' => 'кашмири (ғәрәп)',
        'ks_Arab_IN' => 'кашмири (ғәрәп, Һиндостан)',
        'ks_Deva' => 'кашмири (деванагари)',
        'ks_Deva_IN' => 'кашмири (деванагари, Һиндостан)',
        'ks_IN' => 'кашмири (Һиндостан)',
        'ku' => 'курд',
        'ku_Latn' => 'курд (латиница)',
        'ku_Latn_IQ' => 'курд (латиница, Ираҡ)',
        'ku_Latn_SY' => 'курд (латиница, Сүриә)',
        'ku_Latn_TR' => 'курд (латиница, Төркиә)',
        'ku_SY' => 'курд (Сүриә)',
        'ku_TR' => 'курд (Төркиә)',
        'kw' => 'корн',
        'kw_GB' => 'корн (Бөйөк Британия)',
        'ky' => 'ҡырғыҙ',
        'ky_KG' => 'ҡырғыҙ (Ҡырғыҙстан)',
        'lb' => 'люксембург',
        'lb_LU' => 'люксембург (Люксембург)',
        'lg' => 'ганда',
        'lg_UG' => 'ганда (Уганда)',
        'ln' => 'лингала',
        'ln_AO' => 'лингала (Ангола)',
        'ln_CD' => 'лингала (Конго - Киншаса)',
        'ln_CF' => 'лингала (Үҙәк Африка Республикаһы)',
        'ln_CG' => 'лингала (Конго - Браззавиль)',
        'lo' => 'лаос',
        'lo_LA' => 'лаос (Лаос)',
        'lt' => 'литва',
        'lt_LT' => 'литва (Литва)',
        'lu' => 'луба-катанга',
        'lu_CD' => 'луба-катанга (Конго - Киншаса)',
        'lv' => 'латыш',
        'lv_LV' => 'латыш (Латвия)',
        'mg' => 'малагаси',
        'mg_MG' => 'малагаси (Мадагаскар)',
        'mi' => 'маори',
        'mi_NZ' => 'маори (Яңы Зеландия)',
        'mk' => 'македон',
        'mk_MK' => 'македон (Төньяҡ Македония)',
        'ml' => 'малаялам',
        'ml_IN' => 'малаялам (Һиндостан)',
        'mn' => 'монгол',
        'mn_MN' => 'монгол (Монголия)',
        'mr' => 'маратхи',
        'mr_IN' => 'маратхи (Һиндостан)',
        'ms' => 'малай',
        'ms_BN' => 'малай (Бруней)',
        'ms_ID' => 'малай (Индонезия)',
        'ms_MY' => 'малай (Малайзия)',
        'ms_SG' => 'малай (Сингапур)',
        'mt' => 'мальта',
        'mt_MT' => 'мальта (Мальта)',
        'my' => 'бирма',
        'my_MM' => 'бирма (Мьянма [Бирма])',
        'nb' => 'норвег букмол',
        'nb_NO' => 'норвег букмол (Норвегия)',
        'nb_SJ' => 'норвег букмол (Шпицберген һәм Ян-Майен)',
        'nd' => 'төньяҡ ндебеле',
        'nd_ZW' => 'төньяҡ ндебеле (Зимбабве)',
        'ne' => 'непал',
        'ne_IN' => 'непал (Һиндостан)',
        'ne_NP' => 'непал (Непал)',
        'nl' => 'нидерланд',
        'nl_AW' => 'нидерланд (Аруба)',
        'nl_BE' => 'нидерланд (Бельгия)',
        'nl_BQ' => 'нидерланд (Карибтағы Нидерланд)',
        'nl_CW' => 'нидерланд (Кюрасао)',
        'nl_NL' => 'нидерланд (Нидерланд)',
        'nl_SR' => 'нидерланд (Суринам)',
        'nl_SX' => 'нидерланд (Синт-Мартен)',
        'nn' => 'норвег нюнорск',
        'nn_NO' => 'норвег нюнорск (Норвегия)',
        'no' => 'норвег',
        'no_NO' => 'норвег (Норвегия)',
        'oc' => 'окситан',
        'oc_ES' => 'окситан (Испания)',
        'oc_FR' => 'окситан (Франция)',
        'om' => 'оромо',
        'om_ET' => 'оромо (Эфиопия)',
        'om_KE' => 'оромо (Кения)',
        'or' => 'одия',
        'or_IN' => 'одия (Һиндостан)',
        'os' => 'осетин',
        'os_GE' => 'осетин (Грузия)',
        'os_RU' => 'осетин (Рәсәй)',
        'pa' => 'панджаби',
        'pa_Arab' => 'панджаби (ғәрәп)',
        'pa_Arab_PK' => 'панджаби (ғәрәп, Пакистан)',
        'pa_Guru' => 'панджаби (гурмукхи)',
        'pa_Guru_IN' => 'панджаби (гурмукхи, Һиндостан)',
        'pa_IN' => 'панджаби (Һиндостан)',
        'pa_PK' => 'панджаби (Пакистан)',
        'pl' => 'поляк',
        'pl_PL' => 'поляк (Польша)',
        'ps' => 'пушту',
        'ps_AF' => 'пушту (Афғанстан)',
        'ps_PK' => 'пушту (Пакистан)',
        'pt' => 'португал',
        'pt_AO' => 'португал (Ангола)',
        'pt_BR' => 'португал (Бразилия)',
        'pt_CH' => 'португал (Швейцария)',
        'pt_CV' => 'португал (Кабо-Верде)',
        'pt_GQ' => 'португал (Экваториаль Гвинея)',
        'pt_GW' => 'португал (Гвинея-Бисау)',
        'pt_LU' => 'португал (Люксембург)',
        'pt_MO' => 'португал (Макао, Ҡытай МАР-ы)',
        'pt_MZ' => 'португал (Мозамбик)',
        'pt_PT' => 'португал (Португалия)',
        'pt_ST' => 'португал (Сан-Томе һәм Принсипи)',
        'pt_TL' => 'португал (Тимор-Лесте)',
        'qu' => 'кечуа',
        'qu_BO' => 'кечуа (Боливия)',
        'qu_EC' => 'кечуа (Эквадор)',
        'qu_PE' => 'кечуа (Перу)',
        'rm' => 'романш',
        'rm_CH' => 'романш (Швейцария)',
        'rn' => 'рунди',
        'rn_BI' => 'рунди (Бурунди)',
        'ro' => 'румын',
        'ro_MD' => 'румын (Молдова)',
        'ro_RO' => 'румын (Румыния)',
        'ru' => 'урыҫ',
        'ru_BY' => 'урыҫ (Беларусь)',
        'ru_KG' => 'урыҫ (Ҡырғыҙстан)',
        'ru_KZ' => 'урыҫ (Ҡаҙағстан)',
        'ru_MD' => 'урыҫ (Молдова)',
        'ru_RU' => 'урыҫ (Рәсәй)',
        'ru_UA' => 'урыҫ (Украина)',
        'rw' => 'киньяруанда',
        'rw_RW' => 'киньяруанда (Руанда)',
        'sa' => 'санскрит',
        'sa_IN' => 'санскрит (Һиндостан)',
        'sc' => 'сардин',
        'sc_IT' => 'сардин (Италия)',
        'sd' => 'синдхи',
        'sd_Arab' => 'синдхи (ғәрәп)',
        'sd_Arab_PK' => 'синдхи (ғәрәп, Пакистан)',
        'sd_Deva' => 'синдхи (деванагари)',
        'sd_Deva_IN' => 'синдхи (деванагари, Һиндостан)',
        'sd_IN' => 'синдхи (Һиндостан)',
        'sd_PK' => 'синдхи (Пакистан)',
        'se' => 'төньяҡ саам',
        'se_FI' => 'төньяҡ саам (Финляндия)',
        'se_NO' => 'төньяҡ саам (Норвегия)',
        'se_SE' => 'төньяҡ саам (Швеция)',
        'sg' => 'санго',
        'sg_CF' => 'санго (Үҙәк Африка Республикаһы)',
        'si' => 'сингал',
        'si_LK' => 'сингал (Шри-Ланка)',
        'sk' => 'словак',
        'sk_SK' => 'словак (Словакия)',
        'sl' => 'словен',
        'sl_SI' => 'словен (Словения)',
        'sn' => 'шона',
        'sn_ZW' => 'шона (Зимбабве)',
        'so' => 'сомали',
        'so_DJ' => 'сомали (Джибути)',
        'so_ET' => 'сомали (Эфиопия)',
        'so_KE' => 'сомали (Кения)',
        'so_SO' => 'сомали (Сомали)',
        'sq' => 'албан',
        'sq_AL' => 'албан (Албания)',
        'sq_MK' => 'албан (Төньяҡ Македония)',
        'sr' => 'серб',
        'sr_BA' => 'серб (Босния һәм Герцеговина)',
        'sr_Cyrl' => 'серб (кириллица)',
        'sr_Cyrl_BA' => 'серб (кириллица, Босния һәм Герцеговина)',
        'sr_Cyrl_ME' => 'серб (кириллица, Черногория)',
        'sr_Cyrl_RS' => 'серб (кириллица, Сербия)',
        'sr_Latn' => 'серб (латиница)',
        'sr_Latn_BA' => 'серб (латиница, Босния һәм Герцеговина)',
        'sr_Latn_ME' => 'серб (латиница, Черногория)',
        'sr_Latn_RS' => 'серб (латиница, Сербия)',
        'sr_ME' => 'серб (Черногория)',
        'sr_RS' => 'серб (Сербия)',
        'st' => 'көньяҡ сото',
        'st_LS' => 'көньяҡ сото (Лесото)',
        'st_ZA' => 'көньяҡ сото (Көньяҡ Африка Республикаһы)',
        'su' => 'сундан',
        'su_ID' => 'сундан (Индонезия)',
        'su_Latn' => 'сундан (латиница)',
        'su_Latn_ID' => 'сундан (латиница, Индонезия)',
        'sv' => 'швед',
        'sv_AX' => 'швед (Аланд утрауҙары)',
        'sv_FI' => 'швед (Финляндия)',
        'sv_SE' => 'швед (Швеция)',
        'sw' => 'суахили',
        'sw_CD' => 'суахили (Конго - Киншаса)',
        'sw_KE' => 'суахили (Кения)',
        'sw_TZ' => 'суахили (Танзания)',
        'sw_UG' => 'суахили (Уганда)',
        'ta' => 'тамил',
        'ta_IN' => 'тамил (Һиндостан)',
        'ta_LK' => 'тамил (Шри-Ланка)',
        'ta_MY' => 'тамил (Малайзия)',
        'ta_SG' => 'тамил (Сингапур)',
        'te' => 'телугу',
        'te_IN' => 'телугу (Һиндостан)',
        'tg' => 'тажик',
        'tg_TJ' => 'тажик (Тажикстан)',
        'th' => 'тай',
        'th_TH' => 'тай (Тайланд)',
        'ti' => 'тигринья',
        'ti_ER' => 'тигринья (Эритрея)',
        'ti_ET' => 'тигринья (Эфиопия)',
        'tk' => 'төркмән',
        'tk_TM' => 'төркмән (Төркмәнстан)',
        'tn' => 'тсвана',
        'tn_BW' => 'тсвана (Ботсвана)',
        'tn_ZA' => 'тсвана (Көньяҡ Африка Республикаһы)',
        'to' => 'тонга',
        'to_TO' => 'тонга (Тонга)',
        'tr' => 'төрөк',
        'tr_CY' => 'төрөк (Кипр)',
        'tr_TR' => 'төрөк (Төркиә)',
        'tt' => 'татар',
        'tt_RU' => 'татар (Рәсәй)',
        'ug' => 'уйғыр',
        'ug_CN' => 'уйғыр (Ҡытай)',
        'uk' => 'украин',
        'uk_UA' => 'украин (Украина)',
        'ur' => 'урду',
        'ur_IN' => 'урду (Һиндостан)',
        'ur_PK' => 'урду (Пакистан)',
        'uz' => 'үзбәк',
        'uz_AF' => 'үзбәк (Афғанстан)',
        'uz_Arab' => 'үзбәк (ғәрәп)',
        'uz_Arab_AF' => 'үзбәк (ғәрәп, Афғанстан)',
        'uz_Cyrl' => 'үзбәк (кириллица)',
        'uz_Cyrl_UZ' => 'үзбәк (кириллица, Үзбәкстан)',
        'uz_Latn' => 'үзбәк (латиница)',
        'uz_Latn_UZ' => 'үзбәк (латиница, Үзбәкстан)',
        'uz_UZ' => 'үзбәк (Үзбәкстан)',
        'vi' => 'вьетнам',
        'vi_VN' => 'вьетнам (Вьетнам)',
        'wo' => 'волоф',
        'wo_SN' => 'волоф (Сенегал)',
        'xh' => 'коса',
        'xh_ZA' => 'коса (Көньяҡ Африка Республикаһы)',
        'yi' => 'идиш',
        'yi_UA' => 'идиш (Украина)',
        'yo' => 'йоруба',
        'yo_BJ' => 'йоруба (Бенин)',
        'yo_NG' => 'йоруба (Нигерия)',
        'za' => 'чжуан',
        'za_CN' => 'чжуан (Ҡытай)',
        'zh' => 'ҡытай',
        'zh_CN' => 'ҡытай (Ҡытай)',
        'zh_HK' => 'ҡытай (Гонконг, Ҡытай МАР-ы)',
        'zh_Hans' => 'ҡытай (ябайлаштырылған)',
        'zh_Hans_CN' => 'ҡытай (ябайлаштырылған, Ҡытай)',
        'zh_Hans_HK' => 'ҡытай (ябайлаштырылған, Гонконг, Ҡытай МАР-ы)',
        'zh_Hans_MO' => 'ҡытай (ябайлаштырылған, Макао, Ҡытай МАР-ы)',
        'zh_Hans_MY' => 'ҡытай (ябайлаштырылған, Малайзия)',
        'zh_Hans_SG' => 'ҡытай (ябайлаштырылған, Сингапур)',
        'zh_Hant' => 'ҡытай (традицион)',
        'zh_Hant_HK' => 'ҡытай (традицион, Гонконг, Ҡытай МАР-ы)',
        'zh_Hant_MO' => 'ҡытай (традицион, Макао, Ҡытай МАР-ы)',
        'zh_Hant_MY' => 'ҡытай (традицион, Малайзия)',
        'zh_Hant_TW' => 'ҡытай (традицион, Тайвань)',
        'zh_MO' => 'ҡытай (Макао, Ҡытай МАР-ы)',
        'zh_SG' => 'ҡытай (Сингапур)',
        'zh_TW' => 'ҡытай (Тайвань)',
        'zu' => 'зулу',
        'zu_ZA' => 'зулу (Көньяҡ Африка Республикаһы)',
    ],
];
