<?php

return \deepclone_from_array([
    'classes' => '',
    'objectMeta' => 0,
    'prepared' => [-1],
    'mask' => [false],
    'refs' => [
        1 => [-1],
    ],
    'refMasks' => [
        1 => [false],
    ],
]);
